define(['jquery', 'vendor/backbone', 'vendor/underscore', 'lib/components/base/modal', 'twigjs', 'lib/components/notes/notes', 'lib/components/base/player', 'lib/common/urlparams', 'lib/common/fn', 'vendor/jplayer'], function ($, Backbone, _, Modal, twig, NotesWrapper, Player, url_params, Fn, jp) {
  var CallResult, Notes;

  Notes = NotesWrapper.View.extend({
    noteSwitchToTask: function (e) {
      this.switchToTask({
        currentTarget: e.currentTarget
      });
    }
  });

  CallResult = Backbone.View.extend({
    tagName: "div",
    className: "js-call_result_modal",

    events: {
      'click .{{widget_code}}__call_result__audio_delete': 'removeCallRecord',
      'focus .note-edit__textarea': 'modalCentrify',
      'blur .js-call_result__suggest-contact': 'blurSuggestContactCompany',
      'input .js-call_result__suggest': 'checkSuggestIcon',
      'suggest:loaded .js-call_result__suggest': 'showSuggestLoaded',
      'suggest:changed .js-call_result__suggest': 'changeSuggestIcon',
      'suggest:changed .js-call_result__suggest-contact': 'suggestContactCompanyChanged',
      'suggest:changed .js-call_result__suggest-lead': 'suggestLeadChanged',
      'click #js-modal-accept': 'saveResultStart',

      'click .control--suggest--input': 'suggestInputClick',
      'focus .control--suggest--input': 'suggestInputFocus',

      'click .feed-compose-switcher': 'showSwitcherClick',
      'click .js-tips-item': 'selectTip',

      'mouseover .feed-compose-switcher': 'switcherMouseOver',
      'mouseleave .feed-compose-switcher__tip': 'switcherMouseLeave',
      'mouseover .feed-compose-switcher__tip': 'switcherTipMouseOver',

      'focus .rc-feed__message': 'callResultAdd',
      'blur .rc-feed__message': 'callResultAddFocus',
      'keydown .rc-feed__message': 'keyDownCallRes',
    },

    initialize: function (params) {
      var _this = this;
      this.custom_fields = null;
      this.widget = params.widget;
      this.lang = this.widget.i18n('result_form');
      this.time_delay = Math.floor(new Date() / 1000);

      params.element = params.element || false;
      if (params.element) {
        this.set_element(params.element);
      } else {
        this.element = {
          id: 0,
          element_type: 0,
          entity: {},
          element: {}
        };
      }

      this.user = AMOCRM.constant('user');
      this.deleted = false;
      this.call_info = params.call_info;
      this.call_record = params.call_record;
      this.call_time = params.call_time;
      this.start_time = params.start_time;
      this.need_page_reload = false;
      this.note_type = params.note_type;
      this.is_save = false;
      this.session_id = params.session_id;
      this.result = {
        el: {},
        lead: {}
      };

      if (!AMOCRM.constant('task_types') || !this.custom_fields) {
        $.ajax({
          url: '/private/api/v2/json/accounts/current',
          dataType: 'json'
        }).done(function (res) {
          if (res && res.response && res.response.account && res.response.account.task_types instanceof Array) {
            var task_types = res.response.account.task_types,
                ret = {};
            for (var i = 0; i < task_types.length; i++) {
              task = task_types[i];
              ret[task.id] = {
                id: "key_" + task.id,
                option: '<span class="task_type_select__icon icon icon-inline icon-clock-blue-big"></span>' + task.name
              }
            }
          }
          if (res && res.response && res.response.account && res.response.account.custom_fields && res.response.account.custom_fields.contacts instanceof Array) {
            _this.custom_fields = res.response.account.custom_fields.contacts;
          } else {
            _this.custom_fields = [];
          }
        });
      }

      this.linkSuggestElement(this.element);

      this.modal_obj = new Modal({
        class_name: 'modal-list modal-widget modal-call-result-ringcentral',
        init: _.bind(function ($modal_body) {
          this.modal = $modal_body;
          $modal_body.html(this.$el);

          this.checkIfCardHasChanges();
        }, this),
        widget: this.widget,
        destroy: _.bind(this.destroy, this)
      });
    },

    destroy: function () {
      $('.amo-player--playing .js-icon-stop').trigger('click');
      if(!this.is_save && this.result.el.id && this.note_type != 11){
        this.result.el.id = 0;
        this.result.el.element_type = 0;
      }

      this.addUnsavedTask();
      if(this.call_record.result.state() === 'resolved'){
        this.call_record.result.then(_.bind(function (res) {
          if(res && res.id && res.link){
            this.call_record.url = res.link;
            this.call_record.id = res.id;
          }
          this.saveCall({
            success: function (response) {},
            always: function () {}
          });
        }, this));
      } else {
        this.saveCall({
          success: function (response) {},
          always: function () {}
        });
      }
      this.is_save = false;
    },

    checkIfCardHasChanges: function () {
      if (AMOCRM.data.current_card) {
        if (window.location.pathname.split('/')[2] === 'add') {
          AMOCRM.data.current_card.changes = {};
          AMOCRM.data.current_card.changes.has_changes = true;
        }
        if (AMOCRM.data.current_card.checkChanges()) {
          var elem_type = AMOCRM.data.current_card.element_type,
            element = {
              id: AMOCRM.data.current_card.id,
              element_type: elem_type,
              element: {
                element_id: AMOCRM.data.current_card.id,
                text: AMOCRM.data.current_card.$name[0].value
              }
          };
          AMOCRM.data.current_card.save();

          if (elem_type !== AMOCRM.element_types.leads) {
            this.set_element(element);
            this.linkSuggestElement(element);
          }
          this.render();
        } else {
          this.render();
        }
      } else {
        this.render();
      }
    },

    render: function () {
      var call_statuses = $.extend({}, AMOCRM.call_statuses),
          search_by_phone_ready = $.Deferred(),
          linked_contact_ready = $.Deferred(),
          linked_company_ready = $.Deferred(),
          self = this,
          call_statuses_keys = _.keys(call_statuses);
      call_statuses.amo_call_status_success_conversation.selected = true;
      call_statuses = this.sortStatuses(call_statuses);
      AMOCRM.addLang(this.widget.i18n('result_form'));

      if (this.element && this.element.element) {
        this.element.element = this.element.element;
      } else {
        this.element.element = {};
      }

      if (this.element.element.text) {
        this.element.element.text = Fn.escapeHTML(this.element.element.text);
      }

      if (this.result.lead.text) {
        this.result.lead.text = Fn.escapeHTML(this.result.lead.text);
      }

      this.widget.findByNumber(this.call_info.phone, _.bind(function(result) {
        switch (result.type_string) {
          case 1:
            result.type_string = 'contacts';
            break;
          case 2:
            result.type_string = 'leads';
            break;
          case 3:
            result.type_string = 'companies';
            break;
          case 12:
            result.type_string = 'customers';
            break;
        }
        if(!_.isEmpty(result)){
          $.ajax({url: '/api/v2/' + result.type_string + '?id=' + result.id}).done(_.bind(function (linked_result) {
              if(!_.isEmpty(linked_result._embedded) && !_.isEmpty(linked_result._embedded.items[0])
                && !_.isEmpty(linked_result._embedded.items[0].company)){

                $.ajax({url: '/api/v2/companies?id=' + linked_result._embedded.items[0].company.id}).done(_.bind(function(companyResponse){
                  if(!_.isEmpty(companyResponse._embedded) && !_.isEmpty(companyResponse._embedded.items[0])){
                    result.company = companyResponse._embedded.items[0];
                  }
                  linked_company_ready.resolve();
                } ,this)).fail(function () {
                  linked_company_ready.resolve();
                });
              } else {
                linked_company_ready.resolve();
              }

              if(!_.isEmpty(linked_result._embedded) && !_.isEmpty(linked_result._embedded.items[0])
                && !_.isEmpty(linked_result._embedded.items[0].main_contact)){
                $.ajax({url: '/api/v2/contacts?id=' + linked_result._embedded.items[0].main_contact.id}).done(_.bind(function(contactsResponse){
                  if(!_.isEmpty(contactsResponse._embedded) && !_.isEmpty(contactsResponse._embedded.items[0])){
                    result.contact = contactsResponse._embedded.items[0];
                  }
                  linked_contact_ready.resolve();
                } ,this)).fail(function () {
                  linked_contact_ready.resolve();
                });

              } else {
                linked_contact_ready.resolve();
              }

              search_by_phone_ready.resolve(result);
            }
          ), this).fail(_.bind(function () {
            search_by_phone_ready.resolve({});
          }, this));
        } else {
          linked_contact_ready.resolve();
          linked_company_ready.resolve();
          search_by_phone_ready.resolve(result);

        }
      }));



      $.when(search_by_phone_ready, linked_contact_ready, linked_company_ready).then( _.bind(function(result) {
        var elem = result;
        if (_.isEmpty(this.element.element) && result.id > 0) {
          elem = {
            element: {
              text: elem.name,
              element_id: elem.id
            }
          };
          this.element = elem;
        }
        if(!_.isEmpty(result.phone_entity)){
          this.element.element.name = result.phone_entity.name
        }

        var extra = {companies:{}, contacts: {}, customers: {}, leads:{}};
        var new_extra = {};
        new_extra.name = result.name;

        if(result.phone_entity){
          if(result.phone_entity.type == 1){
            new_extra.contact_name = result.phone_entity.name;
            if(result.company){
              new_extra.company_name = result.company.name;
            }
          } else if(result.phone_entity.type == 3){
            new_extra.company_name = result.phone_entity.name;
            if(result.contact){
              new_extra.contact_name = result.contact.name;
            }
          }

        } else {
          if(result.contact){
            new_extra.contact_name = result.contact.name;
          }
          if(result.company){
            new_extra.company_name = result.company.name;
          }
        }

        this.widget.getTemplate('linked', _.bind( function (template) {
          var main_suggest = template.render({extra: extra});

        // var string_name = this.generateExtraName(extra);
          self.colorsString = {};
         var string_name  = self.parseNames(new_extra, 2);
        var disabled_input = string_name ? true : false;
        this.widget.getTemplate(
          'new_call_result',
          _.bind(function (template) {
            this.$el.html(template.render({
              notes_block: this.widget.render({
                ref: '/tmpl/cards/notes/wrapper.twig'
              }, {
                element_id: this.element.element_id || 1,
                current_user: this.user,
                managers: AMOCRM.constant('managers') || {},
                lang: $.extend({}, AMOCRM.lang, {
                  note_placeholder: this.widget.i18n('result_form').note_placeholder,
                  task_placeholder: this.widget.i18n('result_form').task_placeholder
                }),
                task_types: AMOCRM.constant('task_types') || {},
                file_button: {
                  url: 'where'
                }
              }),
              // suggest: main_suggest,
              suggest: this.widget.render({
                ref: '/tmpl/controls/suggest.twig'
              }, {
                selected: '',
                value_id: this.element.element_id,
                class_name: 'call_result__suggest',
                input_class_name: 'js-call_result__suggest js-call_result__suggest-contact',
                placeholder: '',
                disabled: disabled_input,
                ajax: {
                  url: '/ajax/v1/elements/list',
                  params: 'term=#q#'
                },
                additional_data: 'data-no-filter="y"'
              }),
              lead_suggest: this.widget.render({
                ref: '/tmpl/controls/suggest.twig'
              }, {
                selected: this.result.lead.text,
                value_id: this.result.lead.id,
                class_name: 'call_result__suggest',
                input_class_name: 'js-call_result__suggest js-call_result__suggest-lead',
                placeholder: this.widget.i18n('result_form').lead_placeholder,
                ajax: {
                  url: '/private/ajax/search.php',
                  params: 'type=deals&q=#q#'
                }
              }),
              lead_linked: !!(this.result.lead.id),
              element: this.element,
              record: this.call_record,
              widget_code: 'YOUR_WIDGET_CODE',
              call_time: (this.call_time > 0) ? moment(this.call_time).format('mm:ss') : '00:00',
              phone: this.call_info.phone,
              result_options: call_statuses,
              lang: this.lang,
              link_exist: disabled_input
            }));

            $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').keyup(function (e) {
              if(self.result && self.result.el){

                self.result.el.id = 0;
              }
            });
            $('.button-change-linked').on('click', function (e) {
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').removeAttr('disabled');
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').val('');
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').focus();
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').attr('placeholder', 'Entity name');
              $('.color-text').html('');
              if(self.result && self.result.el){
                self.result.el.id = 0;
              }


              $(e.currentTarget).addClass('hide');
            });
            if(string_name){
              $('.color-text').html(string_name);
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').val('');
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').attr('placeholder', '');
            } else {
              $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').attr('placeholder', 'Entity name');
            }
            this.$el.find('.task-edit__footer').hide();
            this.$el.find('.js-note-add-textarea').empty();
            this.$el.find('.js-task-add-textarea').empty();
            this.$el.find('.js-note-switcher').remove();
            this.$el.find('.task-add-holder').remove();
            this.$el.find('.note-edit__textarea').attr('placeholder', 'Call result');
            this.modal
              .trigger('modal:loaded')
              .trigger('modal:centrify');
            var my_this = this;
            this.call_record.result.done(function (res) {
              this.player = new Player({
                selector: '#vox_recorded_talk',
                $container: my_this.$el
              });
            });

            if (this.element.element_id) {
              this.linkSuggestElement({
                id: this.element.element_id,
                element_type: this.element.type
              });
            }

            this.notes_view = new Notes({
              element_type: this.element.type,
              user: this.user.id,
              el: this.modal.find('.notes-wrapper'),
              no_player: true
            });

            this.notes_view.notes.on('notes:updated', _.bind(function () {
              setTimeout(_.bind(function () {
                this.modal.trigger('modal:centrify');
              }, this), 300);
            }, this));
          }, this)
        );
        }, this));
      }, this));
    },

      keyDownCallRes: function (e) {
          if(e.keyCode === 13){
              $('.rc-feed__message').append('<div><br/></div>');
              $('.rc-feed__message').css({paddingLeft: "0px"});
          }
      },

      callResultAdd: function () {
          $('.js-control-contenteditable-rc').hide();
      },

      callResultAddFocus: function () {
          var val = $('.js-control-contenteditable-input').val();
          if(val == ""){
              $('.js-control-contenteditable-rc').show();
          } else {
              return true;
          }

      },

      showSwitcherClick: function () {
          $('.feed-compose-switcher__tip').show();
      },

      switcherMouseLeave: function () {
          this._set = _.delay(_.bind(function () {
              $('.feed-compose-switcher__tip').hide();
          }, this), 300);
      },

      switcherTipMouseOver: function () {
          clearTimeout(this._set);
      },

      selectTip: function (e) {
          var el = e.target,
              elItems = $('.feed-compose-switcher__tip').find('.tips-item');
          _.map(elItems, function(item) {
              item.classList.remove('tips-item_selected');
          });


          if(!el.classList.contains('tips-item_selected')){

              el.classList.add('tips-item_selected');

              if(el.textContent == 'Call Result'){
                  $('.task-edit__body').hide();
                  $('.feed-compose__inner').show();
                  this.widget.getTemplate('call_result_note', function (template) {
                      $('.feed-compose__inner').html(template.render());
                  });
              }else {
                  $('.feed-compose__inner').hide();
                  $('.task-edit__body').show();
              }
          }
          $('#js-task-edit').text(el.textContent);

          $('.feed-compose-switcher__tip').hide();
      },


      set_element: function (element) {
      this.element = {
        id: element.id || element.element_id,
        element_type: element.element_type || element.type,
        entity: element.entity,
        element: element.element
      };

      this.first_linked_element = this.element;
    },

    addLinkIcon: function (type) {
      this.$el.find('.{{widget_code}}__call_result__common.' + type).addClass('element_linked');
    },

    removeLinkIcon: function (type) {
      this.$el.find('.{{widget_code}}__call_result__common.' + type).removeClass('element_linked');
    },

    checkSuggestIcon: function (e) {
      var $this = $(e.currentTarget),
          type = $this.closest('.{{widget_code}}__call_result__common').hasClass('contact') ? 'contact' : 'lead';

      $this.removeAttr('data-value-id');
      this.removeLinkIcon(type);
      this.result[type === 'contact' ? 'el' : 'lead'] = {};
    },

    get_current_duration: function (record) {
      var _this = this;

      if (record.exist && !this.deleted) {
        record.result.then(function (url) {
          _this.$el.find('.{{widget_code}}__call_result__play_audio').prop('href', url);
          _this.$el.find('.{{widget_code}}__call_info').hide();
        });
      }
    },


    blurSuggestContactCompany: function (e) {
      var $this = $(e.currentTarget),
          new_suggest_value;
    },

    //Убираем событие по клику на инпут
    suggestInputClick: function(e){
      e.stopPropagation();
    },
    //Убираем событие по фокусу инпута
    suggestInputFocus: function(e){
      if(!$('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').val()){
        e.stopPropagation();
      }
    },

    suggestContactCompanyChanged: function (e, $li) {
      $('.button-change-linked').removeClass('hide');
      $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').attr('disabled', 'disabled');
      this.linkSuggestElement({
        id: $li.data('value-id'),
        element_type: $li.data('element_type')
      });

      if(this.colorsString && this.colorsString[$li.data('value-id')]){
        $('.color-text').html(this.colorsString[$li.data('value-id')]);
        $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').attr('placeholder', '');
      }
      $('.{{widget_code}}__call_result__wrapper .js-control--suggest--input-ajax').val('');
    },

    suggestLeadChanged: function (e, $li) {
      this.result.lead = {
        id: $li.data('value-id'),
        element_type: AMOCRM.element_types.leads
      };
    },

    changeSuggestIcon: function (e) {
      this.addLinkIcon($(e.currentTarget).closest('.{{widget_code}}__call_result__common').hasClass('contact') ? 'contact' : 'lead');
    },

    showSuggestLoaded: function (e, data, $ul) {
      var items = [],
          self = this,
          type;

      data = data.response || {};
      this.colorsString = {};
      if (data.items) {
        $.each(data.items, function (e, row) {
          var currentItems = e == 'leads' ? row.other : row;
          items.push({id: 0, text: '<b>' + e.toUpperCase() + '</b>', custom_class: 'js--suggest-item--not-selectable'});
          switch (e) {
            case 'contacts':
              type = 1;
              break;
            case 'leads':
              type = 2;
              break;
            case 'companies':
              type = 3;
              break;
            case 'customers':
              type = 12;
              break;
          }


          $.each(currentItems, function (key, item) {
            items.push({
              id: item.id,
              text: self.parseNames(item, type),
              additional_data: ' data-element_type="' + type + '" '
            });

          });

        });
      }
      $ul.trigger('suggest:reset', [items]);
    },

    parseNames: function(item, type){
      var namesArray = [];
      var colorString = '';

      if(item.name){
        namesArray.push(item.name);
        switch (type) {
          case 1:
            colorString += '<span class="cc-clr">'+ item.name +', </span>'
            break;
          case 2:
          case 12:
            colorString += '<span class="ll-clr">'+ item.name +', </span>'
            break;
          case 3:
            colorString += '<span class="cm-clr">'+ item.name +', </span>'
            break;
        }

      }

      if(item.contact_name && type !== 3){
        namesArray.push(item.contact_name);
        colorString += '<span class="cc-lr">'+ item.contact_name +', </span>'
      }

      if(item.company_name){
        namesArray.push(item.company_name);
        colorString += '<span class="cm-clr">'+ item.company_name +', </span>'
      }

      var lastChar = colorString.slice(-9);
      if (lastChar == ', </span>') {
        colorString = colorString.slice(0, -9);
      }

      this.colorsString[item.id] ='<span>' + colorString + '</span>';
      return colorString;
      // return namesArray.join(', ');
    },

    linkSuggestElement: function (element) {
      if (typeof element === 'object') {
        this.result.el = element;
      }

      this.addLinkIcon('contact');
    },

    setExtra: function(extra, data){

      var info = {name: data.name, id: data.id};
      switch (data.type) {
        case 1:
          extra.contacts = info;
          break;
        case 2:
          extra.leads = info;
          break;
        case 3:
          extra.companies = info;
          break;
        case 12:
          extra.customers = info;
      }
      return extra;
    },

    generateExtraName: function(extra){
      var nameArray = [];
      if(extra.leads.name){
        nameArray.push(extra.leads.name);
      }
      if(extra.customers.name){
        nameArray.push(extra.customers.name);
      }
      if(extra.contacts.name){
        nameArray.push(extra.contacts.name);
      }
      if(extra.companies.name){
        nameArray.push(extra.companies.name);
      }
      return nameArray.join(', ');
    },

    saveResultStart: function () {
      this.is_save = true;
      this.$el.find('#js-modal-accept').trigger('button:save:start');

      this.createNewElements({
        error: _.bind(function () {
          this.$el.find('#js-modal-accept').trigger('button:load:stop');

          this.widget.notifers.show_message_error({
            text: 'cant create new element'
          });
        }, this),
        complete: _.bind(function (result) {
          this.createSlaveElements();
        }, this)
      });
    },

    updateContact: function (contact_def) {
      var api_dir = (this.result.el.element_type === 1) ? 'contacts' : 'company',
          contact_id = this.result.el.id;

      this.apiCall({
        type: 'contacts/links',
        api_dir: false,
        method: 'get',
        data: {
          id: contact_id
        },
        error: function () {},
        success: function (data) {
          var leads = [this.result.lead.id],
              c_data;
          if (data && data.links instanceof Array) {
            data.links.forEach(function (el) {
              if (el && el.lead_id) {
                leads.push(el.lead_id);
              }
            });
          }
          c_data = {
            id: contact_id,
            linked_leads_id: leads,
            last_modified: (+new Date())
          }
          if (this.result.el.name) {
            c_data.name = this.result.el.name;
          }
          this.apiCall({
            type: 'contacts',
            api_dir: api_dir || false,
            method: 'update',
            data: c_data,
            error: function () {},
            success: function () {
              contact_def.resolve();
            }
          });
        }
      });
    },

    addUnsavedTask: function (finishCallback) {
      if (this.notes_view && this.notes_view.add_task_model) {
        if (this.notes_view.add_task_model.get('has_changes')) {
          this.notes_view.add_task.saveClick();
        }
      }

      if (_.isFunction(finishCallback)) {
        finishCallback.apply(this);
      }

      return this;
    },

    apiCall: function (params) {
      var data = {
          request: {}
        },
          method = params.method || 'add',
          url,
          dir = params.api_dir || params.type;

      data['request'][params.type] = {};
      data['request'][params.type][method] = [params.data];

      url = '/private/api/v2/json/' + dir + (method === 'get' ? '' : '/set');
      if (method === "get" && params.type === "contacts/links") {
        if (params && params.data && params.data.id) {
          url = url + "?contacts_link=" + params.data.id;
        }
      }

      $.ajax({
        url: url,
        data: data,
        method: (method === 'get' ? 'GET' : 'POST')
      }).always(_.bind(function (data) {
        if (data) {
          data = data.response;
        }

        if (params.type !== "contacts/links" && (!data || !data[params.type] || !data[params.type][method]) && method !== 'update') {
          params.error.call(this);
        } else {
          this.need_page_reload = true;

          params.success.call(this, params.type !== "contacts/links" ? {
            id: data && data[params.type] && data[params.type][method] instanceof Array && data[params.type][method][0] && data[params.type][method][0].id,
            element_type: AMOCRM.element_types[params.type]
          } : data);
        }
      }, this));
    },

    createNewElements: function (params) {
      var callbacks = {
          complete: params.complete || function () {},
          error: params.error || function () {}
        },
          contact_def = $.Deferred(),
          lead_def = $.Deferred(),
          contact_id,
          this_custom_fields = this.custom_fields,
          lead_id;

      if (!this.result.lead) {
        this.result.lead = {};
      }

      if (!this.result.el) {
        this.result.el = {};
      }

        var fields = this_custom_fields,
            cPhone,
            _this = this,
            ePhone,
            fPhone;

        if ((this.call_info && ((this.call_info.element_info && this.call_info.element_info.phone) || this.call_info.phone)) ||
          (this.call_record && this.call_record.to)) {

          if (this.call_info && this.call_info.element_info && this.call_info.element_info.phone) {
            cPhone = this.call_info.element_info.phone;
          } else if (this.call_info && this.call_info.phone) {
            cPhone = this.call_info.phone;
          } else {
            cPhone = this.call_record.to;
          }
          if (cPhone && typeof cPhone === "string" && cPhone[0] === "+") {
            cPhone = cPhone.substr(1);
          }
        }

      if (!fPhone && fields instanceof Array) {
        fields.forEach(function (el) {
          if (el && el.code === "PHONE" && el.id) {
            fPhone = el.id;
            $.each(el.enums, function (key, value) {
              if(value == 'WORK'){
                ePhone = key;
              }
            });


          }
        });
      }

        if (!this.result.el.id && cPhone) {
            var found = false;
            var newContactName;


          
          if(!_.isEmpty(_this.$el.find('.js-call_result__suggest-contact').val())){
            newContactName = _this.$el.find('.js-call_result__suggest-contact').val();
          }

          if (!found && newContactName) {
            _this.apiCall({
              type: 'leads',
              data: {
                name: '',
              },
              error: function () {
                lead_def.resolve({})
              },
              success: function (data) {
                lead_def.resolve(data.id)
              }
            });
            
            lead_def.then(function (leadId) {
              _this.apiCall({
                type: 'contacts',
                data: {
                  name: newContactName,
                  custom_fields: [{
                    id: fPhone,
                    values: [{
                      value: cPhone,
                      enum: "WORK"
                    }]
                  }],
                  linked_leads_id: [leadId]
                },
                error: function () {},
                success: function (data) {
                  //Если есть сделка, то прикрепим звонок к ней
                  if(leadId){
                    lead_id = leadId;
                    _this.result.el = {
                      id: leadId,
                      element_type: 2
                    };
                  } else {
                    _this.result.el = {
                      id: data.id,
                      element_type: data.element_type
                    };
                  }
                  contact_id = data.id;
                  contact_def.resolve();
                }
              });
            });

          } else {
            // _this.updateContact(contact_def);
            contact_def.resolve();
          }

        } else {
          //Если нужно прикрепить к существующему контакту или компании, то нужно добавить в нее еще и новый телефон
          if((_this.result.el.element_type == 1 || _this.result.el.element_type == 3) && _this.result.el.id){
            var currentEntity = _this.result.el.element_type == 1 ? 'contacts' : 'companies',
                existedPhonesReady = $.Deferred(),
                existedPhones = [];

            $.ajax({
              url: '/api/v2/' + currentEntity + '?id=' + _this.result.el.id,
              method: 'get'
            }).done(function (response) {
              if(response && response._embedded && response._embedded.items[0]){
                var item = response._embedded.items[0];
                $.each(item.custom_fields, function (key, value) {
                  if(value.id == fPhone){
                    existedPhones = value.values;
                  }
                })
              }
              var isNeedAddNewPhone = true;
              $.each(existedPhones, function (key, value) {
                var phoneString = value.value.slice(-5);
                var newPhoneString = cPhone.slice(-5);
                if(phoneString == newPhoneString){
                  isNeedAddNewPhone = false;
                }
              });
              if(isNeedAddNewPhone){
                existedPhonesReady.resolve(existedPhones);
              } else {
                existedPhonesReady.reject();
              }

            }).fail(function () {
              existedPhonesReady.resolve(existedPhones);
            });

            existedPhonesReady.then(function (phones) {
              var currentPhones = phones;
              currentPhones.push({
                enum: ePhone,
                value: cPhone
              });
              $.ajax({
                url: '/api/v2/' + currentEntity,
                method: 'post',
                data: {
                  update: [{
                    id: _this.result.el.id,
                    updated_at: Math.floor(Date.now() / 1000),
                    custom_fields: [
                      {
                        id: fPhone,
                        values: currentPhones
                      }
                    ]
                  }]
                }
              }).done(function () {
                contact_def.resolve();
              }).fail(function () {
                contact_def.resolve();
              });
            }).fail(function () {
              contact_def.resolve();
            });

          } else {
            contact_def.resolve();
          }
        }

      $.when(contact_def).done(function () {
        cb = 'complete';

        callbacks[cb]({
          data: {
            contact_id: contact_id,
            lead_id: [lead_id]
          }
        });
      });
    },

    createSlaveElements: function (params) {
      this.saveNotes(params);
      this.$el.find('#js-call-result-cancel').click();
    },

    reloadPage: function () {
      if (AMOCRM.data.current_card) {
        AMOCRM.router.navigate(url_params.setQueryParam({
          reload: Fn.randHex()
        }), {
          trigger: true,
          replace: true
        });
      } else {
        $(document).trigger('list:reload');
      }
    },

    saveNotes: function (params) {
      if (this.notes_view.add_task_model.get('has_changes')) {
        this.notes_view.add_task.saveClick();
      }
      if (this.notes_view.notes.length > 0) {
        this.notes_view.notes.each(function (model) {
          var model_params = model.get('params'),
              task_element = this.result.lead && this.result.lead.id ? this.result.lead : this.result.el;

          model_params.changed = {
            text: model_params.text,
            main_user: model_params.main_user.id,
            type: model_params.type,
            date: model.get('date')
          };

          this.widget.addToCard({text: model_params.text,
            id: task_element.id,
            element_type: task_element.element_type,
            type: model_params.type,
            time: model.get('date')});

          model.addCommit({
            success: function (data) {},
            error: function (data) {}
          });
        }, this);

        this.need_page_reload = true;
      }
    },

    removeCallRecord: function () {
      this.$el.find('#vox_recorded_talk').remove();
      this.$el.find('.{{widget_code}}__call_info').remove();
      this.modal.trigger('modal:centrify');

      this.deleted = true;

      if (this.call_record) {
        this.call_record.url = '';
      }
    },

    saveCall: function (params) {
      //your function to save calls
    },

    sortStatuses: function(statuses){
      var sortMap = {
        amo_call_status_success_conversation: 1,
        amo_call_status_leave_message: 2,
        amo_call_status_fail_busy: 3,
        amo_call_status_success_recall: 4,
        amo_call_status_fail_not_phoned: 5,
        amo_call_status_success_not_in_stock: 6,
        amo_call_status_fail_wrong_number: 7
      };
      var statusResult = [];
      $.each(sortMap, function (key, value) {
        statusResult.push(statuses[key]);
      });
      return statusResult;
    },

    modalCentrify: function () {
      this.modal.trigger('modal:centrify');
    }
  });

  return CallResult;
});
